@@ -5,12 +5,16 @@
 #'
 #' @param envname The name of, or path to, a conda environment.
 #'
-#' @param conda The path to a `conda` executable. Use `"auto"` to allow `reticulate` to
-#'   automatically find an appropriate `conda` binary. See **Finding Conda** for more details.
+#' @param conda The path to a `conda` executable. Use `"auto"` to allow
+#'   `reticulate` to automatically find an appropriate `conda` binary. See
+#'   **Finding Conda** for more details.
 #'
-#' @param packages A character vector, indicating package names which should be installed or removed.
+#' @param packages A character vector, indicating package names which should be
+#'   installed or removed. Use `python=<version>` to request the installation
+#'   of a specific version of Python.
 #'
-#' @param pip Boolean; use `pip` when downloading or installing packages? Defaults to `FALSE`.
+#' @param pip Boolean; use `pip` when downloading or installing packages?
+#'   Defaults to `FALSE`.
 #'
 #' @param ... Optional arguments, reserved for future expansion.
 #'
@@ -31,8 +35,9 @@
 #' following locations:
 #' 
 #' 1. The location specified by the `reticulate.conda_binary` \R option;
-#' 2. The program `PATH`;
-#' 3. A set of pre-defined locations where Conda is typically installed.
+#' 2. The [miniconda_path()] location (if it exists);
+#' 3. The program `PATH`;
+#' 4. A set of pre-defined locations where Conda is typically installed.
 #'
 #' @export
 conda_list <- function(conda = "auto") {
@@ -65,11 +70,22 @@ conda_list <- function(conda = "auto") {
   # strip out anaconda cloud prefix (not valid json)
   if (length(conda_envs) > 0 && grepl("Anaconda Cloud", conda_envs[[1]], fixed = TRUE))
     conda_envs <- conda_envs[-1]
+  
+  # parse conda info
+  info <- fromJSON(conda_envs)
 
   # convert to json
-  conda_envs <- fromJSON(conda_envs)$envs
+  conda_envs <- info$envs
   conda_envs <- Filter(file.exists, conda_envs)
-
+  
+  # normalize and remove duplicates (seems necessary on Windows as Anaconda
+  # may report both short-path and long-path versions of the same environment)
+  conda_envs <- unique(normalizePath(conda_envs))
+  
+  # remove root conda environment as it shouldnnt count as environment and
+  # it is not correct to install packages into it.
+  conda_envs <- conda_envs[!conda_envs == normalizePath(info$root_prefix)]
+  
   # return an empty data.frame when no envs are found
   if (length(conda_envs) == 0L) {
     return(data.frame(
@@ -79,10 +95,6 @@ conda_list <- function(conda = "auto") {
     )
   }
 
-  # normalize and remove duplicates (seems necessary on Windows as Anaconda
-  # may report both short-path and long-path versions of the same environment)
-  conda_envs <- unique(normalizePath(conda_envs))
-
   # build data frame
   name <- character()
   python <- character()
@@ -110,6 +122,14 @@ conda_list <- function(conda = "auto") {
 #'   When specified, the `forge` argument is ignored. If you need to
 #'   specify multiple channels, including the Conda Forge, you can use
 #'   `c("conda-forge", <other channels>)`.
+#'   
+#' @param python_version The version of Python to be used in this Conda
+#'   environment. The associated Python package from Conda will be requested
+#'   as `python={python_version}`. When `NULL`, the default `python` package
+#'   will be used instead. For example, use `python_version = "3.6"` to request
+#'   that the Conda environment be created with a copy of Python 3.6. This
+#'   argument will be ignored if `python` is specified as part of the `packages`
+#'   argument, for backwards compatibility.
 #'
 #' @rdname conda-tools
 #'
@@ -117,16 +137,27 @@ conda_list <- function(conda = "auto") {
 #'
 #' @export
 conda_create <- function(envname = NULL,
-                         packages = "python",
+                         packages = NULL,
                          forge = TRUE,
                          channel = character(),
-                         conda = "auto") {
+                         conda = "auto",
+                         python_version = NULL)
+{
 
   # resolve conda binary
   conda <- conda_binary(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
+  
+  # resolve packages argument
+  if (!any(grepl("^python", packages))) {
+    python_package <- if (is.null(python_version))
+      "python"
+    else
+      sprintf("python=%s", python_version)
+    packages <- c(python_package, packages)
+  }
 
   # create the environment
   args <- conda_args("create", envname, packages)
@@ -182,15 +213,15 @@ conda_remove <- function(envname, packages = NULL, conda = "auto") {
 #'   specify multiple channels, including the Conda Forge, you can use
 #'   `c("conda-forge", <other channels>)`.
 #'
-#' @param pip_ignore_installed Ignore installed versions when using pip. This is
-#'   `TRUE` by default so that specific package versions can be installed even
-#'   if they are downgrades. The `FALSE` option is useful for situations where
-#'   you don't want a pip install to attempt an overwrite of a conda binary
-#'   package (e.g. SciPy on Windows which is very difficult to install via pip
-#'   due to compilation requirements).
+#' @param pip_ignore_installed Ignore installed versions when using pip
+#'   (defaults to `FALSE`). Set this to `TRUE` so that specific package versions
+#'   can be installed even if they are downgrades. The `FALSE` option is useful
+#'   for situations where you don't want a pip install to attempt an overwrite of
+#'   a conda binary package (e.g. SciPy on Windows which is very difficult to
+#'   install via pip due to compilation requirements).
 #'   
 #' @param pip_options An optional character vector of additional command line
-#'   arguments to be passed to `pip` if `pip` is used.
+#'   arguments to be passed to `pip`. Only relevant when `pip = TRUE`.
 #'
 #' @rdname conda-tools
 #'
@@ -208,27 +239,48 @@ conda_install <- function(envname = NULL,
                           python_version = NULL,
                           ...)
 {
+  check_forbidden_install("Python packages")
+  
   # resolve conda binary
   conda <- conda_binary(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
 
-  # honor request for specific Python
-  python_package <- "python"
-  if (!is.null(python_version))
-    python_package <- paste(python_package, python_version, sep = "=")
+  # honor request for specific version of Python package
+  python_package <- if (is.null(python_version))
+    NULL
+  else if (grepl("[><=]", python_version))
+    paste0("python", python_version)
+  else
+    sprintf("python=%s", python_version)
 
-  # check if the environment exists, and create it on demand if needed.
-  # if the environment does already exist, but a version of Python was
-  # requested, attempt to install that in the existing environment
-  # (effectively re-creating it if the Python version differs)
-  python <- tryCatch(conda_python(envname = envname, conda = conda), error = identity)  
+  # check to see if we already have a valid Python installation for
+  # this conda environment
+  python <- tryCatch(
+    conda_python(envname = envname, conda = conda),
+    error = identity
+  )
   
+  # if this conda environment doesn't seem to exist, auto-create it
   if (inherits(python, "error") || !file.exists(python)) {
-    conda_create(envname, packages = python_package, conda = conda)
+    
+    conda_create(
+      envname = envname,
+      packages = python_package %||% "python",
+      forge = forge,
+      channel = channel,
+      conda = conda
+    )
+    
     python <- conda_python(envname = envname, conda = conda)
-  } else if (!is.null(python_package)) {
+    
+  }
+  
+  # if the user has requested a specific version of Python, ensure that
+  # version of Python is installed into the requested environment
+  # (should be no-op if that copy of Python already installed)
+  if (!is.null(python_version)) {
     args <- conda_args("install", envname, python_package)
     status <- system2(conda, shQuote(args))
     if (status != 0L) {
@@ -239,8 +291,18 @@ conda_install <- function(envname = NULL,
   }
 
   # delegate to pip if requested
-  if (pip)
-    return(pip_install(python, packages, pip_options = pip_options))
+  if (pip) {
+    
+    result <- pip_install(
+      python = python,
+      packages = packages,
+      pip_options = pip_options,
+      ignore_installed = pip_ignore_installed
+    )
+    
+    return(result)
+    
+  }
   
   # otherwise, use conda
   args <- conda_args("install", envname)
@@ -351,6 +413,10 @@ find_conda <- function() {
   if (!is.null(conda))
     return(conda)
   
+  # if miniconda is installed, use it
+  if (miniconda_exists())
+    return(miniconda_conda())
+  
   # if there is a conda executable on the PATH, use it
   conda <- Sys.which("conda")
   if (nzchar(conda))
@@ -501,3 +567,11 @@ conda_list_packages <- function(envname = NULL, conda = "auto", no_pip = TRUE) {
   )
   
 }
+
+conda_installed <- function() {
+  condabin <- tryCatch(conda_binary(), error = identity)
+  if (inherits(condabin, "error"))
+    return(FALSE)
+  else
+    return(TRUE)
+}
