@@ -74,38 +74,15 @@ disable_conversion_scope <- function(object) {
   TRUE
 }
 
-new_stack <- function() {
-
-  (function() {
-
-    .data <- list()
-
-    methods <- list(
-      clear  = function() { .data <<- character() },
-      data   = function() { .data },
-      empty  = function() { length(.data) == 0 },
-      length = function() { length(.data) },
-      push   = function(line) { .data[[length(.data) + 1]] <<- line },
-      peek   = function() { .data[[length(.data)]] },
-      pop    = function() { .data <<- utils::head(.data, n = -1) },
-      set    = function(data) { .data <<- data }
-    )
-
-    list2env(methods)
-
-  })()
-
-}
-
-py_compile_eval <- function(code, compile_mode = "single") {
+py_compile_eval <- function(code, compile_mode = "single", capture = TRUE) {
 
   builtins <- import_builtins(convert = TRUE)
   sys <- import("sys", convert = TRUE)
 
   # allow 'globals' and 'locals' to both point at main module, so that
   # evaluated code updates references there as well
-  globals <- py_eval("globals()", convert = FALSE)
-  locals <- globals
+  main <- import_main(convert = FALSE)
+  globals <- locals <- py_get_attr(main, "__dict__")
 
   # Python's command compiler complains if the only thing you submit
   # is a comment, so detect that case first
@@ -119,13 +96,18 @@ py_compile_eval <- function(code, compile_mode = "single") {
   # compile and eval the code -- use 'single' to auto-print statements
   # as they are evaluated, or 'exec' to avoid auto-print
   compiled <- builtins$compile(code, '<string>', compile_mode)
-  output <- py_capture_output(builtins$eval(compiled, globals, locals))
+  if (capture) {
+    output <- py_capture_output(builtins$eval(compiled, globals, locals))
+  } else {
+    builtins$eval(compiled, globals, locals)
+    output <- NULL
+  }
 
   # save the value that was produced
   .globals$py_last_value <- py_last_value()
 
   # py_capture_output can append an extra trailing newline, so remove it
-  if (grepl("\n{2,}$", output))
+  if (!is.null(output) && grepl("\n{2,}$", output))
     output <- sub("\n$", "", output)
 
   # and return
@@ -135,7 +117,7 @@ py_compile_eval <- function(code, compile_mode = "single") {
 py_last_value <- function() {
   tryCatch(
     py_eval("_", convert = FALSE),
-    error = function(e) r_to_py(NULL)
+    error = function(e) py_none()
   )
 }
 
@@ -180,32 +162,32 @@ path_prepend <- function(entries) {
 # to a heuristic (note: false positives are possible but we can accept
 # those in the contexts where this function is used)
 file_same <- function(lhs, rhs) {
-  
+
   # check if paths are identical as-is
   if (identical(lhs, rhs))
     return(TRUE)
-  
+
   # check if paths are identical after normalization
   lhs <- normalizePath(lhs, winslash = "/", mustWork = FALSE)
   rhs <- normalizePath(rhs, winslash = "/", mustWork = FALSE)
   if (identical(lhs, rhs))
     return(TRUE)
-  
+
   # check if file info is the same
-  lhsi <- c(file.info(lhs, extra_cols = FALSE))
-  rhsi <- c(file.info(rhs, extra_cols = FALSE))
+  lhsi <- suppressWarnings(c(file.info(lhs, extra_cols = FALSE)))
+  rhsi <- suppressWarnings(c(file.info(rhs, extra_cols = FALSE)))
   fields <- c("size", "isdir", "mode", "mtime", "ctime")
   if (identical(lhsi[fields], rhsi[fields]))
     return(TRUE)
-  
+
   # checks failed; return FALSE
   FALSE
-  
+
 }
 
 # normalize a path without following symlinks
 canonical_path <- function(path) {
-  
+
   # on windows we normalize the whole path to avoid
   # short path components leaking in
   if (is_windows()) {
@@ -216,7 +198,7 @@ canonical_path <- function(path) {
       basename(path)
     )
   }
-  
+
 }
 
 enumerate <- function(x, f, ...) {
@@ -225,3 +207,88 @@ enumerate <- function(x, f, ...) {
     f(n[[i]], x[[i]], ...)
   })
 }
+
+is_interactive <- function() {
+  
+  # detect case where RStudio is being used, but reticulate is being
+  # executed as part of a user's .Rprofile (for example). in this case,
+  # we aren't really interactive as we are unable to respond to readline
+  # requests, and so we want to avoid miniconda prompts
+  rstudio <- !is.na(Sys.getenv("RSTUDIO", unset = NA))
+  gui <- .Platform$GUI
+  if (rstudio && !identical(gui, "RStudio"))
+    return(FALSE)
+  
+  # otherwise, use base implementation
+  interactive()
+  
+}
+
+is_r_cmd_check <- function() {
+  
+  # if NOT_CRAN is set, this is likely devtools::check() -- allow it
+  not_cran <- Sys.getenv("NOT_CRAN", unset = NA)
+  if (identical(not_cran, "true"))
+    return(FALSE)
+  
+  # if _R_CHECK_PACKAGE_NAME_ is set, then we must be running R CMD check
+  package_name <- Sys.getenv("_R_CHECK_PACKAGE_NAME_", unset = NA)
+  if (!is.na(package_name))
+    return(TRUE)
+  
+  # does not appear to be R CMD check
+  FALSE
+  
+}
+
+stack <- function(mode = "list") {
+  
+  .data <- vector(mode)
+  
+  object <- list(
+    
+    set = function(data) {
+      .data <<- data
+    },
+    
+    push = function(...) {
+      dots <- list(...)
+      for (data in dots) {
+        if (is.null(data))
+          .data[length(.data) + 1] <<- list(NULL)
+        else
+          .data[[length(.data) + 1]] <<- data
+      }
+    },
+    
+    pop = function() {
+      item <- .data[[length(.data)]]
+      length(.data) <<- length(.data) - 1
+      item
+    },
+    
+    peek = function() {
+      .data[[length(.data)]]
+    },
+    
+    contains = function(data) {
+      data %in% .data
+    },
+    
+    empty = function() {
+      length(.data) == 0
+    },
+    
+    clear = function() {
+      .data <<- list()
+    },
+    
+    data = function() {
+      .data
+    }
+    
+  )
+  
+  object
+  
+}
