@@ -59,7 +59,11 @@ bool loadLibrary(const std::string& libPath, void** ppLib, std::string* pError)
 #ifdef _WIN32
   *ppLib = (void*)::LoadLibraryEx(libPath.c_str(), NULL, 0);
 #else
-  *ppLib = ::dlopen(libPath.c_str(), RTLD_NOW|RTLD_GLOBAL);
+  if (libPath == "NA") {
+    *ppLib = ::dlopen(NULL, RTLD_NOW|RTLD_GLOBAL);
+  } else {
+    *ppLib = ::dlopen(libPath.c_str(), RTLD_NOW|RTLD_GLOBAL);
+  }
 #endif
   if (*ppLib == NULL)
   {
@@ -171,14 +175,23 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
 
   LOAD_PYTHON_SYMBOL(Py_Initialize)
   LOAD_PYTHON_SYMBOL(Py_IsInitialized)
+  LOAD_PYTHON_SYMBOL(Py_GetVersion)
   LOAD_PYTHON_SYMBOL(Py_AddPendingCall)
   LOAD_PYTHON_SYMBOL(PyErr_SetInterrupt)
   LOAD_PYTHON_SYMBOL(PyExc_KeyboardInterrupt)
   LOAD_PYTHON_SYMBOL(Py_IncRef)
   LOAD_PYTHON_SYMBOL(Py_DecRef)
+  LOAD_PYTHON_SYMBOL(PyObject_Size)
+  LOAD_PYTHON_SYMBOL(PyObject_Type);
+  LOAD_PYTHON_SYMBOL(PyObject_GetAttr)
+  LOAD_PYTHON_SYMBOL(PyObject_HasAttr)
+  LOAD_PYTHON_SYMBOL(PyObject_SetAttr)
   LOAD_PYTHON_SYMBOL(PyObject_GetAttrString)
   LOAD_PYTHON_SYMBOL(PyObject_HasAttrString)
   LOAD_PYTHON_SYMBOL(PyObject_SetAttrString)
+  LOAD_PYTHON_SYMBOL(PyObject_GetItem)
+  LOAD_PYTHON_SYMBOL(PyObject_SetItem)
+  LOAD_PYTHON_SYMBOL(PyObject_DelItem)
   LOAD_PYTHON_SYMBOL(PyTuple_Size)
   LOAD_PYTHON_SYMBOL(PyTuple_GetItem)
   LOAD_PYTHON_SYMBOL(PyTuple_New)
@@ -194,6 +207,7 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
   LOAD_PYTHON_SYMBOL(PyErr_NormalizeException)
   LOAD_PYTHON_SYMBOL(PyErr_ExceptionMatches)
   LOAD_PYTHON_SYMBOL(PyErr_GivenExceptionMatches)
+  LOAD_PYTHON_SYMBOL(PyObject_Print)
   LOAD_PYTHON_SYMBOL(PyObject_Str)
   LOAD_PYTHON_SYMBOL(PyObject_Dir)
   LOAD_PYTHON_SYMBOL(PyByteArray_Size)
@@ -201,7 +215,6 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
   LOAD_PYTHON_SYMBOL(PyByteArray_AsString)
   LOAD_PYTHON_SYMBOL(PyCallable_Check)
   LOAD_PYTHON_SYMBOL(PyRun_StringFlags)
-  LOAD_PYTHON_SYMBOL(Py_CompileString)
   LOAD_PYTHON_SYMBOL(PyEval_EvalCode)
   LOAD_PYTHON_SYMBOL(PyModule_GetDict)
   LOAD_PYTHON_SYMBOL(PyImport_AddModule)
@@ -228,6 +241,7 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
   LOAD_PYTHON_SYMBOL(PyFunction_Type)
   LOAD_PYTHON_SYMBOL(PyModule_Type)
   LOAD_PYTHON_SYMBOL(PyType_Type)
+  LOAD_PYTHON_SYMBOL(PyProperty_Type)
   LOAD_PYTHON_SYMBOL(PyComplex_FromDoubles)
   LOAD_PYTHON_SYMBOL(PyComplex_RealAsDouble)
   LOAD_PYTHON_SYMBOL(PyComplex_ImagAsDouble)
@@ -242,7 +256,11 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
   LOAD_PYTHON_SYMBOL(PyGILState_Ensure)
   LOAD_PYTHON_SYMBOL(PyGILState_Release)
   LOAD_PYTHON_SYMBOL(PyThreadState_Next)
-
+  LOAD_PYTHON_SYMBOL(PyObject_CallMethod)
+  LOAD_PYTHON_SYMBOL(PySequence_GetItem)
+  LOAD_PYTHON_SYMBOL(PyObject_IsTrue)
+  LOAD_PYTHON_SYMBOL(PyCapsule_Import)
+  
   // PyUnicode_AsEncodedString may have several different names depending on the Python
   // version and the UCS build type
   std::vector<std::string> names;
@@ -253,6 +271,7 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
     return false;
 
   if (python3) {
+    LOAD_PYTHON_SYMBOL(Py_GetProgramFullPath)
 
     // Debug versions of Python will provide PyModule_Create2TraceRefs,
     // while release versions will provide PyModule_Create
@@ -275,12 +294,14 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
     LOAD_PYTHON_SYMBOL(PyUnicode_FromString)
     LOAD_PYTHON_SYMBOL_AS(PyLong_AsLong, PyInt_AsLong)
     LOAD_PYTHON_SYMBOL_AS(PyLong_FromLong, PyInt_FromLong)
+    LOAD_PYTHON_SYMBOL(Py_CompileStringExFlags)
   } else {
     if (is64bit) {
       LOAD_PYTHON_SYMBOL_AS(Py_InitModule4_64, Py_InitModule4)
     } else {
       LOAD_PYTHON_SYMBOL(Py_InitModule4)
     }
+    LOAD_PYTHON_SYMBOL_AS(Py_GetProgramFullPath, Py_GetProgramFullPath_v2)
     LOAD_PYTHON_SYMBOL(PyString_AsStringAndSize)
     LOAD_PYTHON_SYMBOL(PyString_FromStringAndSize)
     LOAD_PYTHON_SYMBOL(PyString_FromString)
@@ -290,13 +311,14 @@ bool LibPython::loadSymbols(bool python3, std::string* pError)
     LOAD_PYTHON_SYMBOL(PyInt_AsLong)
     LOAD_PYTHON_SYMBOL(PyInt_FromLong)
     LOAD_PYTHON_SYMBOL(PyCObject_AsVoidPtr)
+    LOAD_PYTHON_SYMBOL(Py_CompileString)
   }
   LOAD_PYTHON_SYMBOL(PyCapsule_New)
   LOAD_PYTHON_SYMBOL(PyCapsule_GetPointer)
   LOAD_PYTHON_SYMBOL(PyCapsule_SetContext)
   LOAD_PYTHON_SYMBOL(PyCapsule_GetContext)
   LOAD_PYTHON_SYMBOL(Py_BuildValue)
-
+  
   return true;
 }
 
@@ -359,4 +381,3 @@ bool import_numpy_api(bool python3, std::string* pError) {
 
 
 } // namespace libpython
-
