@@ -125,3 +125,73 @@ test_that("single-row data.frames with rownames can be converted", {
   expect_equal(c(before), c(after))
 
 })
+
+test_that("Time zones are respected if available", {
+  skip_if_no_pandas()
+  
+  pd <- import("pandas", convert = FALSE)
+  
+  before <- pd$DataFrame(list('TZ' = pd$Series(
+    c(
+      pd$Timestamp('20130102003020', tz = 'US/Pacific'),
+      pd$Timestamp('20130102003020', tz = 'CET'),
+      pd$Timestamp('20130102003020', tz = 'UTC'),
+      pd$Timestamp('20130102003020', tz = 'Hongkong')
+    )
+  )))
+  
+  converted <- py_to_r(before)
+  after <- r_to_py(converted)
+  
+  # check if both are the same in *local* timezone
+  expect_equal(py_to_r(before), py_to_r(after))
+  
+})
+
+test_that("NaT is converted to NA", {
+  skip_if_no_pandas()
+  
+  pd <- import("pandas", convert = FALSE)
+  np <- import("numpy")
+  
+  before <- pd$DataFrame(pd$Series(
+    c(
+      pd$Timestamp(NULL),
+      pd$Timestamp(np$nan)
+    )
+  ))
+  
+  converted <- py_to_r(before)
+  after <- r_to_py(converted)
+
+  expect_equal(py_to_r(before), py_to_r(after))
+  
+})
+
+test_that("pandas NAs are converted to R NAs", {
+  skip_if_no_pandas()
+  
+  pd <- import("pandas", convert = FALSE)
+  
+  py_run_string("
+import pandas as pd
+df = pd.DataFrame({'a': [1, 2, 3], 'b': [10, 20, pd.NA]})
+")
+  
+  expect_true(is.na(py$df$b[3]))
+  
+  pdNA <- py_to_r(py_get_attr(pd, "NA"))
+  expect_true(is.na(pdNA))
+  
+})
+
+test_that("categorical NAs are handled", {
+  skip_if_no_pandas()
+  
+  df <- data.frame(x = factor("a", NA))
+  pdf <- r_to_py(df)
+  rdf <- py_to_r(pdf)
+  attr(rdf, "pandas.index") <- NULL
+  expect_equal(df, rdf)
+  
+})
